package pen_flowchart;

import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
//import java.io.IOException;
import java.util.List;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

//import com.itextpdf.text.DocumentException;

public abstract class Parts {
	static Parts empty;
	protected Parts left;
	protected Parts right;
	private String str;
	private int x1, x2, y1, y2;
	private boolean dirty;
	protected static int size = 5;

	static{
		empty = new Parts_Null();
		empty.setLeft(null);
	}

	Parts()
	{
		left = empty;
		right = null;
		dirty = false;
	}

	public void makeClean()
	{
		dirty = false;
	}
	public void makeDirty()
	{
		dirty = true;
	}
	public boolean isDirty() // 呼ぶとdirtyはクリアされる
	{
		boolean rtnv = dirty;
		dirty = false;
		return rtnv;
	}

	abstract public void setSize(Graphics g, FontMetrics fm);
	abstract public int getWidth();
	abstract public int getHeight();

	public boolean isRecursiveEnd()
	{
		return false;
	}

	public boolean isRecursiveBegin()
	{
		return false;
	}

	public void setText(String s)
	{
		str = new String(s);
	}

	public String getText()
	{
		return str;
	}

	public void setX1(int v) { x1 = v; }
	public void setX2(int v) { x2 = v; }
	public void setY1(int v) { y1 = v; }
	public void setY2(int v) { y2 = v; }
	public void setP1(Point p){ x1 = p.x; y1 = p.y; }
	public void setP2(Point p){ x2 = p.x; y2 = p.y; }
	public int getX1() {return x1; }
	public int getX2() {return x2; }
	public int getY1() {return y1; }
	public int getY2() {return y2; }

	public boolean inside(int x, int y)
	{
		return getY1() <= y && getY2() >= y &&
				getX1() <= x && getX2() >= x;
	}


	public Parts getLeft()
	{
		return left;
	}

	public Parts getRight()
	{
		return right;
	}

	public void setLeft(Parts p)
	{
		left = p;
	}

	public void setRight(Parts p)
	{
		if(right != null) right = p;
	}

	public void connectLeft(Parts p)
	{
		p.left = left;
		left = p;
	}

	// 分岐の処理がめどい
	public void connectRight(Parts p)
	{
		if(right != null)
		{
			p.left = left;
			p.right = right;
			right = p;
		}
	}

	public Parts getBlockEnd()
	{
		return this;
	}

	public Parts calcSize(Graphics g, FontMetrics fm, Point p, Point p1, Point p2)
	{
		setSize(g, fm);
		int x1 = p.x - getWidth() / 2 - 2;
		int x2 = p.x + getWidth() / 2 + 2;
		if(x1 < p1.x) p1.x = x1;
		if(x2 > p2.x) p2.x = x2;
		p.y += getHeight();
		if(p2.y < p.y) p2.y = p.y;
		if(getLeft() == null || isRecursiveEnd())
			return this;
		return getLeft().calcSize(g, fm, p, p1, p2);
	}

	abstract public Parts paint(Graphics g, FontMetrics fm, Point p);
//	abstract public Parts paintPdf(com.itextpdf.text.pdf.PdfContentByte cb, int height, Graphics g, FontMetrics fm, Point p);

	public void makeBarList(List<Parts> list)
	{
		list.add(this);
		if(getLeft() != null) getLeft().makeBarList(list);
		if(getRight() != null) getRight().makeBarList(list);
	}
	public void drawString(String s, Graphics g, FontMetrics fm, int x1, int y1, int x2, int y2)
	{
		if(s == null) return;
		Rectangle2D rect = fm.getStringBounds(s, g);
		g.drawString(s, (x1 + x2) /2 - (int)(rect.getWidth() /2),
				(y1 + y2) / 2 - (int)(rect.getHeight() / 2) + fm.getAscent());

	}
/*
	public void drawStringPdf(com.itextpdf.text.pdf.PdfContentByte cb, int height, String s, Graphics g, FontMetrics fm, int x1, int y1, int x2, int y2)

	{
		if(s == null) return;
		Rectangle2D rect = fm.getStringBounds(s, g);
		int x = (x1 + x2) /2 - (int)(rect.getWidth() /2);
		int y =  height - (y1 + y2) / 2 - (int)(rect.getHeight() / 2) + fm.getDescent();
		cb.beginText();
		cb.setTextMatrix(x, y);
       try {
			cb.setFontAndSize(com.itextpdf.text.pdf.BaseFont.createFont(
					"HeiseiKakuGo-W5",
					"UniJIS-UCS2-H",
					com.itextpdf.text.pdf.BaseFont.NOT_EMBEDDED), 12);
		} catch (DocumentException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		cb.setColorFill(com.itextpdf.text.BaseColor.BLACK);
		cb.showText(s);
		cb.setColorFill(com.itextpdf.text.BaseColor.WHITE);
		cb.endText();
	}
*/
	abstract public Parts appendTo(StringBuilder sb, int indent_level);
	protected void makeIndent(StringBuilder sb, int indent_level)
	{
		for(int i = 0; i < indent_level; i++)
			sb.append("  | ");
	}

	public boolean dispMenu(final Flowchart chart, MouseEvent e)
	{
		JPopupMenu popup = new JPopupMenu();
		JMenuItem item = new JMenuItem("編集");
		item.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if(editMe())
					chart.repaint();
			}});
		popup.add(item);
		item = new JMenuItem("削除");
		item.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if(deleteMe(chart))
					chart.repaint();
			}});
		popup.add(item);
		item = new JMenuItem("カット");
		item.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent arg0) {
				if(cutMe(chart))
					chart.repaint();
			}});
		popup.add(item);
		popup.show(e.getComponent(), e.getX(), e.getY());
		return true;
	}

	public boolean deleteMe(Flowchart chart)
	{
		Parts parent = chart.findParent(this);
		if(parent != null)
		{
			parent.setLeft(getBlockEnd().getLeft().getLeft());
			getBlockEnd().setLeft(null);
			chart.makeDirty();
//			left = null;
			return true;
		}
		return false;
	}

	public boolean cutMe(Flowchart chart)
	{
		if(deleteMe(chart))
		{
			chart.moveToClipboard(this);
			chart.makeDirty();
			return true;
		}
		return false;
	}

	public boolean editMe()
	{
		return false;
	}
	abstract Parts makeXML(Document doc, Element parent);

	static public void setSize(int s)
	{
		size = s;
	}

	static public int getSize()
	{
		return size;
	}

	static public void resetSize()
	{
		size = 5;
	}

	public void paint_variables(Graphics g, FontMetrics fm, Point p0, VariablePanel variablePanel) {
		// define only in parts_terminal
	}

}
