package pen_flowchart;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

//import com.itextpdf.text.pdf.PdfContentByte;

public class Parts_Input extends Parts {
	int text_width, text_height;
	String var;

	public Parts_Input()
	{
		super();
		setValue("≪変数≫");
	}

	public void setValue(String s)
	{
		var = new String(s);
		setText(var + "を入力");
	}

	@Override
	public int getHeight() {
		return text_height + size * 2;
	}

	@Override
	public int getWidth() {
		return text_width + size * 4;
	}

	@Override
	public Parts paint(Graphics g, FontMetrics fm, Point p) {
		setX1(p.x - getWidth() / 2);
		setX2(p.x + getWidth() / 2);
		setY1(p.y);
		setY2(p.y + getHeight());
		int x[] = {getX1(), getX2(), getX2(), getX1()};
		int y[] = {getY1() + size, getY1() - size, getY2(), getY2()};
		g.setColor(Color.WHITE);
		g.fillPolygon(x, y, 4);
		g.setColor(Color.BLACK);
		drawString(getText(),g,fm,getX1(),getY1(),getX2(),getY2());
		g.drawPolygon(x,y,4);
		p.y = getY2();
		if(getLeft() == null || getLeft().isRecursiveEnd())
			return this;
		return getLeft().paint(g, fm, p);
	}
/*
	@Override
	public Parts paintPdf(PdfContentByte cb, int height, Graphics g,
			FontMetrics fm, Point p) {
		setX1(p.x - getWidth() / 2);
		setX2(p.x + getWidth() / 2);
		setY1(p.y);
		setY2(p.y + getHeight());
		int x[] = {getX1(), getX2(), getX2(), getX1()};
		int y[] = {getY1() + 5, getY1() - 5, getY2(), getY2()};
		g.setColor(Color.WHITE);
		g.fillPolygon(x, y, 4);
		g.setColor(Color.BLACK);
		drawStringPdf(cb, height, getText(),g,fm,getX1(),getY1(),getX2(),getY2());
		g.drawPolygon(x,y,4);
		p.y = getY2();
		if(getLeft() == null || getLeft().isRecursiveEnd())
			return this;
		return getLeft().paintPdf(cb, height, g, fm, p);
	}
*/
	@Override
	public void setSize(Graphics g, FontMetrics fm) {
		text_width = text_height = 0;
		if(getText() != null)
		{
			Rectangle2D rect = fm.getStringBounds(getText(), g);
			text_width = (int)rect.getWidth();
			text_height = (int)rect.getHeight();
		}
		if(text_width < size * 6) text_width = size * 6;
		if(text_height < size * 2) text_height = size * 2;
	}

	@Override
	public Parts appendTo(StringBuilder sb, int indentLevel) {
		makeIndent(sb, indentLevel);
		sb.append(var + " を入力する\n");
		if(getLeft() != null) return getLeft().appendTo(sb, indentLevel);
		return this;
	}

	@Override
	public boolean editMe()
	{
		EditDialog dialog = new EditDialog();
		dialog.setVisible(true);
		return isDirty();
	}

	private class EditDialog extends MyDialog
	{
		private static final long serialVersionUID = 1L;
		JButton btnOK, btnCancel;
		JTextField text;

		public EditDialog() {
			super();
			this.setTitle("入力の編集");
			JPanel panel = new JPanel();
			panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
			text = new JTextField(40);
			grabFocus(text);
			text.setText(var);
			btnOK = new JButton("OK");
			btnOK.addActionListener(this);
			getRootPane().setDefaultButton(btnOK);
			btnCancel = new JButton("キャンセル");
			btnCancel.addActionListener(this);
			panel.add(new JLabel("変数"));
			panel.add(text);
			panel.add(btnOK);
			panel.add(btnCancel);
			getContentPane().add(panel);
			pack();
			setLocationRelativeTo(null);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() == btnOK)
			{
				setValue(Converter.Zen2Han(text.getText()));
				makeDirty();
				dispose();
			}
			else if(e.getSource() == btnCancel)
				dispose();
		}
	}

	@Override
	Parts makeXML(Document doc, Element parent) {
		Element elem = doc.createElement("input");
		elem.setAttribute("var", var);
		parent.appendChild(elem);
		if(left != null && !left.isRecursiveEnd()) return left.makeXML(doc, parent);
		return this;
	}

}
