package pen_flowchart;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

//import com.itextpdf.text.pdf.PdfContentByte;


public class Parts_Substitute extends Parts {
	int text_width, text_height;
	String var, val;

	public void setValue(String s1, String s2)
	{
		var = new String(s1);
		val = new String(s2);
		setText(var + " ← " + val);
	}

	public Parts_Substitute()
	{
		super();
		setValue("≪変数≫","≪値≫");
	}

	@Override
	public int getHeight() {
		return text_height + size * 2;
	}

	@Override
	public int getWidth() {
		return text_width + size * 4;
	}

	@Override
	public Parts paint(Graphics g, FontMetrics fm, Point p) {
		setX1(p.x - getWidth() / 2);
		setX2(p.x + getWidth() / 2);
		setY1(p.y);
		setY2(p.y + getHeight());
		g.setColor(Color.WHITE);
		g.fillRect(getX1(), getY1(), getWidth(), getHeight());
		g.setColor(Color.BLACK);
		drawString(getText(),g,fm,getX1(),getY1(),getX2(),getY2());
		g.drawRect(getX1(), getY1(), getWidth(), getHeight());
		p.y = getY2();
		if(getLeft() == null || getLeft().isRecursiveEnd())
			return this;
		return getLeft().paint(g, fm, p);
	}
/*
	@Override
	public Parts paintPdf(PdfContentByte cb, int height, Graphics g,
			FontMetrics fm, Point p) {
		setX1(p.x - getWidth() / 2);
		setX2(p.x + getWidth() / 2);
		setY1(p.y);
		setY2(p.y + getHeight());
		g.setColor(Color.WHITE);
		g.fillRect(getX1(), getY1(), getWidth(), getHeight());
		g.setColor(Color.BLACK);
		drawStringPdf(cb, height, getText(),g,fm,getX1(),getY1(),getX2(),getY2());
		g.drawRect(getX1(), getY1(), getWidth(), getHeight());
		p.y = getY2();
		if(getLeft() == null || getLeft().isRecursiveEnd())
			return this;
		return getLeft().paintPdf(cb, height, g, fm, p);
	}
*/
	@Override
	public void setSize(Graphics g, FontMetrics fm) {
		text_width = text_height = 0;
		if(getText() != null)
		{
			Rectangle2D rect = fm.getStringBounds(getText(), g);
			text_width = (int)rect.getWidth();
			text_height = (int)rect.getHeight();
		}
		if(text_width < size * 6) text_width = size * 6;
		if(text_height < size * 2) text_height = size * 2;
	}

	@Override
	public Parts appendTo(StringBuilder sb, int indentLevel) {
		makeIndent(sb, indentLevel);
		sb.append(getText());
		sb.append("\n");
		if(getLeft() != null) return getLeft().appendTo(sb, indentLevel);
		return this;
	}
	@Override
	public boolean editMe()
	{
		EditDialog dialog = new EditDialog(this);
		dialog.setVisible(true);
		return isDirty();
	}

	private class EditDialog extends MyDialog
	{
		private static final long serialVersionUID = 1L;
		JButton btnOK, btnCancel;
		JTextField text1, text2;

		public EditDialog(Parts_Substitute p) {
			super();
//			parts = p;
			this.setTitle("代入の編集");
			JPanel panel = new JPanel();
			panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
			text1 = new JTextField(40);
			grabFocus(text1);
			text1.setText(var);
			text2 = new JTextField(40);
			grabFocus(text2);
			text2.setText(val);
			btnOK = new JButton("OK");
			btnOK.addActionListener(this);
			getRootPane().setDefaultButton(btnOK);
			btnCancel = new JButton("キャンセル");
			btnCancel.addActionListener(this);
			panel.add(new JLabel("変数"));
			panel.add(text1);
			panel.add(new JLabel("値"));
			panel.add(text2);
			panel.add(btnOK);
			panel.add(btnCancel);
			getContentPane().add(panel);
			pack();
			setLocationRelativeTo(null);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() == btnOK)
			{
				setValue(Converter.Zen2Han(text1.getText()),
						Converter.Zen2Han(text2.getText()));
				makeDirty();
				dispose();
			}
			else if(e.getSource() == btnCancel)
				dispose();
		}
	}

	@Override
	Parts makeXML(Document doc, Element parent) {
		Element elem = doc.createElement("substitute");
		elem.setAttribute("var", var);
		elem.setAttribute("val", val);
		parent.appendChild(elem);
		if(left != null && !left.isRecursiveEnd()) return left.makeXML(doc, parent);
		return this;
	}
}
