package pen_flowchart;
import java.awt.Cursor;
import java.awt.Desktop;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;
import javax.swing.filechooser.FileNameExtensionFilter;


import pen.MainGUI;


public class PenFlowchart extends JFrame implements ActionListener{
	private static final long serialVersionUID = 1L;
	private static final String version = "2.10";
	private PenFlowchartPanel targetpanel;
	private JMenuItem menuItemNew, menuItemOpen, menuItemSave,
		menuItemSaveas,  menuItemQuit, menuItemAbout,
		menuItemSaveEps //menuItemSavePdf,
		,menuItemSavePng
		//menuItemRun;
		;
	private File currentDirectory;
	private String filename_f;
	private MainGUI Pen;

	public PenFlowchart(String[] args)
	{
		super();

		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(new WindowAdapter(){
			@Override
			public void windowClosing(WindowEvent e)
			{
				if(targetpanel.isDirty())
				{
					int save = askSave();
					if(save == JOptionPane.CANCEL_OPTION)return;
					if(save == JOptionPane.YES_OPTION && !save()) return;
				}
				dispose();
			}

			@Override
			public void windowClosed(WindowEvent e)
			{
				System.exit(0);
			}
		});
		setIconImage(Toolkit.getDefaultToolkit().createImage(getClass().getResource("pen-flowchart.png")));
		targetpanel = new PenFlowchartPanel(this);

		getContentPane().add(targetpanel);
		pack();
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		JMenu menu = new JMenu("ファイル(F)");
		menu.setMnemonic(KeyEvent.VK_F);
		menuBar.add(menu);
		menuItemNew = new JMenuItem("新規(N)");
		menuItemNew.addActionListener(this);
		menuItemNew.setMnemonic(KeyEvent.VK_N);
		menuItemNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, KeyEvent.CTRL_DOWN_MASK));
		menu.add(menuItemNew);
		menuItemOpen = new JMenuItem("開く(O)");
		menuItemOpen.addActionListener(this);
		menuItemOpen.setMnemonic(KeyEvent.VK_O);
		menuItemOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_DOWN_MASK));
		menu.add(menuItemOpen);
		menuItemSave = new JMenuItem("保存(S)");
		menuItemSave.addActionListener(this);
		menuItemSave.setMnemonic(KeyEvent.VK_S);
		menuItemSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, KeyEvent.CTRL_DOWN_MASK));
		menu.add(menuItemSave);
		menuItemSaveas = new JMenuItem("別名で保存(A)");
		menuItemSaveas.addActionListener(this);
		menuItemSaveas.setMnemonic(KeyEvent.VK_A);
		menu.add(menuItemSaveas);
		menu.addSeparator();
		menuItemSaveEps = new JMenuItem("EPS形式で出力");
		menuItemSaveEps.addActionListener(this);
		menuItemSaveEps.setMnemonic(KeyEvent.VK_E);
		menu.add(menuItemSaveEps);
		menuItemSavePng = new JMenuItem("PNG形式で出力");
		menuItemSavePng.addActionListener(this);
		menuItemSavePng.setMnemonic(KeyEvent.VK_P);
		menu.add(menuItemSavePng);
		/*
		menuItemSavePdf = new JMenuItem("PDF形式で出力");
		menuItemSavePdf.addActionListener(this);
		menuItemSavePdf.setMnemonic(KeyEvent.VK_F);
		menu.add(menuItemSavePdf);
		*/
		menu.addSeparator();
		menuItemQuit = new JMenuItem("終了(X)");
		menuItemQuit.addActionListener(this);
		menuItemQuit.setMnemonic(KeyEvent.VK_X);
		menuItemQuit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, KeyEvent.CTRL_DOWN_MASK));
		menu.add(menuItemQuit);

		menu = new JMenu("ヘルプ(H)");
		menu.setMnemonic(KeyEvent.VK_H);
		menuBar.add(menu);
		menuItemAbout = new JMenuItem("About...");
		menuItemAbout.addActionListener(this);
		menuItemAbout.setMnemonic(KeyEvent.VK_A);
		menu.add(menuItemAbout);

		currentDirectory = null;
		filename_f = null;
		setTitle();
		Pen = new MainGUI(args, false);
		Pen.CreateGUI(false, targetpanel);
	}

	public void setTitle() {
		String s = "PenFlowchart ver" + version;
		if(filename_f != null) s = filename_f + " - " + s;
		if(targetpanel.isDirty())
			s = "*" + s;
		setTitle(s);
	}

	public void refreshPen(String s)
	{
		Pen.refreshCode(s);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		PenFlowchart gui_frame = new PenFlowchart(args);
		gui_frame.setSize(gui_frame.getPreferredSize());
		gui_frame.setVisible(true);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if(source == menuItemQuit)
		{
			int save = askSave();
			if(save == JOptionPane.CANCEL_OPTION)return;
			if(save == JOptionPane.NO_OPTION || save())
				dispose();
		}
		else if(source == menuItemNew)
		{
			int save = askSave();
			if(save == JOptionPane.CANCEL_OPTION)return;
			if(save == JOptionPane.NO_OPTION || save())
			{
				targetpanel.reset();
				filename_f = null;
				setTitle();
			}
		}
		else if(source == menuItemOpen)
		{
			if(targetpanel.isDirty())
			{
				int save = askSave();
				if(save == JOptionPane.CANCEL_OPTION) return;
				if(save == JOptionPane.YES_OPTION && !save())
					return;
			}
			if(open())
			{
				setTitle();
				targetpanel.makeCode();
			}
		}
		else if(source == menuItemSave)
			save();
		else if(source == menuItemSaveas)
			saveas();
		else if(source == menuItemAbout)
			about();
		else if(source == menuItemSaveEps)
			saveEps();
		else if(source == menuItemSavePng)
			savePng();
//		else if(source == menuItemSavePdf)
//			savePdf();
//		else if(source == menuItemRun)
//			runProgram();
	}

//	public String getCode()
//	{
//		return codepanel.getText();
//	}

//	private void runProgram() {
//		new MainGUI(null, false, this);
//	}

	private boolean saveas()
	{
		JFileChooser fc = new JFileChooser();
		FileNameExtensionFilter filter =
			new FileNameExtensionFilter("PENFlowchart (*.penf)", "PENF");
		fc.setDialogTitle("保存");
		fc.setCurrentDirectory(currentDirectory);
		fc.setApproveButtonText("保存");
		if(filename_f != null)fc.setSelectedFile(new File(filename_f));
		fc.setFileFilter(filter);
		int rtnv = fc.showSaveDialog(this.getParent());
		if(rtnv == JFileChooser.APPROVE_OPTION)
		{
			currentDirectory = fc.getCurrentDirectory();
			File file = fc.getSelectedFile();
			filename_f = file.getPath();
			int pos = file.getName().lastIndexOf('.'); // 拡張子がなければ.penを補う
			if(pos == -1) filename_f = filename_f + ".penf";
			targetpanel.save(filename_f);
			return true;
		}
		return false;
	}

	private boolean save()
	{
		if(filename_f == null) return saveas();
		else return targetpanel.save(filename_f);
	}

	private boolean open()
	{
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("開く");
		fc.setCurrentDirectory(currentDirectory);
		if(filename_f != null)fc.setSelectedFile(new File(filename_f));
		fc.setFileFilter(new FileNameExtensionFilter("PenFlowchart (*.penf)", "penf"));
//		fc.addChoosableFileFilter(new FileNameExtensionFilter("Pen (*.pen)", "pen"));
		fc.setAcceptAllFileFilterUsed(false);
		int rtnv = fc.showOpenDialog(this.getParent());
		if(rtnv == JFileChooser.APPROVE_OPTION)
		{
			currentDirectory = fc.getCurrentDirectory();
			File file = fc.getSelectedFile();
			String filename = file.getPath();
			if(targetpanel.load(filename))
			{
				filename_f = filename;
				targetpanel.makeClean();
				return true;
			}
//			int pos = file.getName().lastIndexOf('.'); // 拡張子がなければ.penを補う
//			if(pos == -1) filename = filename + ".penf";
		}
		return false;
	}


	private void about()
	{
		MyDialog dialog = new aboutDialog();
		dialog.setVisible(true);
	}

	private class aboutDialog extends MyDialog
	{
		private static final long serialVersionUID = 1L;
		private JButton btnOK;

		private class URIJLabel extends JLabel
		{
			private static final long serialVersionUID = 1L;
			URI uri;
			MouseListener listener = new MouseListener(){

				@Override
				public void mouseClicked(MouseEvent e) {
					try {
						Desktop.getDesktop().browse(((URIJLabel)e.getSource()).getURI());
					} catch (IOException e1) {
						// TODO 自動生成された catch ブロック
						e1.printStackTrace();
					}
				}

				@Override
				public void mouseEntered(MouseEvent e) {
				}

				@Override
				public void mouseExited(MouseEvent e) {
				}

				@Override
				public void mousePressed(MouseEvent e) {
				}

				@Override
				public void mouseReleased(MouseEvent e) {
				}};
			public URIJLabel(String Uri)
			{
				super("<html><a href=\""+Uri+"\">"+Uri+"</a></html>");
				try {
					uri = new URI(Uri);
				} catch (URISyntaxException e) {
					e.printStackTrace();
				}
				addMouseListener(listener);
				setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			}
			public URI getURI()
			{
				return uri;
			}
		}
		public aboutDialog()
		{
			this.setTitle("About...");
			setIconImage(Toolkit.getDefaultToolkit().createImage(getClass().getResource("pen-flowchart.png")));
			JPanel panel = new JPanel();
			JPanel subpanel = new JPanel();
			subpanel.setLayout(new BoxLayout(subpanel, BoxLayout.Y_AXIS));
			subpanel.add(new JLabel("<html><h1>PenFlowchart</h1>" +
					"<pre>Version " + version+"\n"+
					"Copyright (c) 2003-2012\n"+
					"中村 亮太\n" +
					"西田 知博\n"+
					"松浦 敏雄\n"+
					"兼宗 進\n"+
					"中西 渉" +
					"</pre></html>"));
			subpanel.add(new URIJLabel("http://watayan.net/prog/"));
			panel.add(subpanel);
			btnOK = new JButton("OK");
			btnOK.addActionListener(this);
			this.getRootPane().setDefaultButton(btnOK);
			panel.add(btnOK);
			getContentPane().add(panel);
			pack();
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() == btnOK)
				dispose();
		}
	}


	private int askSave()
	{
		if(targetpanel.isDirty())
			return JOptionPane.showConfirmDialog(getContentPane(), "フローチャートは変更されています。保存しますか？","確認", JOptionPane.YES_NO_CANCEL_OPTION);
		return JOptionPane.NO_OPTION;	// Dirtyじゃないならセーブの必要なし
	}

	private boolean saveEps()
	{
		String filename_e = null;
		JFileChooser fc = new JFileChooser();
		FileNameExtensionFilter filter =
			new FileNameExtensionFilter("EPS (*.eps)", "EPS");
		fc.setDialogTitle("EPS形式出力");
		fc.setCurrentDirectory(currentDirectory);
		fc.setApproveButtonText("出力");
		fc.setFileFilter(filter);
		int rtnv = fc.showSaveDialog(this.getParent());
		if(rtnv == JFileChooser.APPROVE_OPTION)
		{
			currentDirectory = fc.getCurrentDirectory();
			File file = fc.getSelectedFile();
			filename_e = file.getPath();
			int pos = file.getName().lastIndexOf('.'); // 拡張子がなければ.penを補う
			if(pos == -1) filename_e = filename_e + ".eps";
			try {
				FileOutputStream s = new FileOutputStream(filename_e);
				s.write(targetpanel.getEps(s, targetpanel.variablePanel).getBytes());
				s.close();
				return true;
			} catch (FileNotFoundException e1) {
				JOptionPane.showConfirmDialog(getParent(), "File I/O Error", "PEN",
						JOptionPane.OK_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
			} catch (IOException e1) {
				JOptionPane.showConfirmDialog(getParent(), "File I/O Error", "PEN",
						JOptionPane.OK_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
			}
		}
		return false;
	}

	private boolean savePng()
	{
		String filename_e = null;
		JFileChooser fc = new JFileChooser();
		FileNameExtensionFilter filter =
			new FileNameExtensionFilter("PNG (*.png)", "PNG");
		fc.setDialogTitle("PNG形式出力");
		fc.setCurrentDirectory(currentDirectory);
		fc.setApproveButtonText("出力");
		fc.setFileFilter(filter);
		int rtnv = fc.showSaveDialog(this.getParent());
		if(rtnv == JFileChooser.APPROVE_OPTION)
		{
			currentDirectory = fc.getCurrentDirectory();
			File file = fc.getSelectedFile();
			filename_e = file.getPath();
			int pos = file.getName().lastIndexOf('.'); // 拡張子がなければ.penを補う
			if(pos == -1) filename_e = filename_e + ".png";
			targetpanel.savePng(filename_e);
		}
		return false;
	}

/*
	private boolean savePdf() {
		String filename_f = null;
		JFileChooser fc = new JFileChooser();
		FileNameExtensionFilter filter =
			new FileNameExtensionFilter("PDF (*.pdf)", "PDF");
		fc.setDialogTitle("PDF形式出力");
		fc.setCurrentDirectory(currentDirectory);
		fc.setApproveButtonText("出力");
		fc.setFileFilter(filter);
		int rtnv = fc.showOpenDialog(this.getParent());
		if(rtnv == JFileChooser.APPROVE_OPTION)
		{
			currentDirectory = fc.getCurrentDirectory();
			File file = fc.getSelectedFile();
			filename_f = file.getPath();
			int pos = file.getName().lastIndexOf('.'); // 拡張子がなければ.penを補う
			if(pos == -1) filename_f = filename_f + ".pdf";
			try {
				FileOutputStream s = new FileOutputStream(filename_f);
				targetpanel.writePdf(s);
				s.close();

				return true;
			} catch (FileNotFoundException e1) {
				JOptionPane.showConfirmDialog(getParent(), "File I/O Error", "PEN",
						JOptionPane.OK_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return false;
	}
*/
}
